"""Each js assertion (console.assert) interface implementations.

Mainly following interfaces are defined:

- assert_equal
    JavaScript assertion interface for equal condition.
- assert_not_equal
    JavaScript assertion interface for not equal condition.
- assert_true
    JavaScript assertion interface for true condition.
- assert_false
    JavaScript assertion interface for false condition.
- assert_arrays_equal
    JavaScript assertion interface for Array values equal condition.
- assert_arrays_not_equal
    JavaScript assertion interface for Array values not equal condition.
- assert_dicts_equal
    JavaScript assertion interface for Dictionary values equal condition.
- assert_dicts_not_equal
    JavaScript assertion interface for Dictionary values not equal
    condition.
- assert_defined
    JavaScript assertion interface for defined (not undefined)
    value condition.
- assert_undefined
    JavaScript assertion interface for undefined value condition.
"""

from typing import Any
from typing import Tuple

from apysc._html.debug_mode import add_debug_info_setting


@add_debug_info_setting(  # type: ignore[misc]
    module_name=__name__)
def assert_equal(
        left: Any, right: Any, *, msg: str = '') -> None:
    """
    JavaScript assertion interface for equal condition.

    Notes
    -----
    - If specified values are types of Array (or list),
        then this interface calls the assert_arrays_equal
        function instead.
    - If specified values are types of Dictionary (or dict),
        then this interface calls the assert_dicts_equal instead.

    Parameters
    ----------
    left : *
        Left-side value to compare.
    right : *
        Right-side value to compare.
    msg : str, optional
        Message to display when assertion failed.

    References
    ----------
    - assert_equal and assert_not_equal interfaces document
        - https://simon-ritchie.github.io/apysc/assert_equal_and_not_equal.html  # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> int_1: ap.Int = ap.Int(10)
    >>> int_2: ap.Int = ap.Int(10)
    >>> ap.assert_equal(int_1, int_2)
    """
    import apysc as ap
    from apysc._string import string_util
    for value in (left, right):
        if _value_type_is_array(value=value):
            assert_arrays_equal(left=left, right=right, msg=msg)
            return
    for value in (left, right):
        if _value_type_is_dict(value=value):
            assert_dicts_equal(left=left, right=right, msg=msg)
            return

    _trace_info(
        interface_label='assert_equal', left=left, right=right)

    left_str, right_str = _get_left_and_right_strs(
        left=left, right=right)

    msg = string_util.escape_str(string=msg)
    expression: str = (
        f'console.assert({left_str} === {right_str}, "{msg}");'
    )
    ap.append_js_expression(expression=expression)


@add_debug_info_setting(  # type: ignore[misc]
    module_name=__name__)
def assert_not_equal(
        left: Any, right: Any, *, msg: str = '') -> None:
    """
    JavaScript assertion interface for not equal condition.

    Notes
    -----
    - If specified values are types of Array (or list),
        then this interface calls the assert_arrays_not_equal
        function instead.
    - If specified values are types of Dictionary (or dict),
        this interface calls the assert_dicts_not_equal
        function instead.

    Parameters
    ----------
    left : *
        Left-side value to compare.
    right : *
        Right-side value to compare.
    msg : str, optional
        Message to display when assertion failed.

    References
    ----------
    - assert_equal and assert_not_equal interfaces document
        - https://simon-ritchie.github.io/apysc/assert_equal_and_not_equal.html  # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> int_1: ap.Int = ap.Int(10)
    >>> int_2: ap.Int = ap.Int(11)
    >>> ap.assert_not_equal(int_1, int_2)
    """
    import apysc as ap
    from apysc._string import string_util
    for value in (left, right):
        if _value_type_is_array(value=value):
            assert_arrays_not_equal(left=left, right=right, msg=msg)
            return
    for value in (left, right):
        if _value_type_is_dict(value=value):
            assert_dicts_not_equal(left=left, right=right, msg=msg)
            return

    _trace_info(
        interface_label='assert_not_equal',
        left=left,
        right=right)
    left_str, right_str = _get_left_and_right_strs(
        left=left, right=right)

    msg = string_util.escape_str(string=msg)
    expression: str = (
        f'console.assert({left_str} !== {right_str}, "{msg}");'
    )
    ap.append_js_expression(expression=expression)


@add_debug_info_setting(  # type: ignore[misc]
    module_name=__name__)
def assert_true(
        value: Any, *, type_strict: bool = True, msg: str = '') -> None:
    """
    JavaScript assertion interface for true condition.

    Parameters
    ----------
    value : *
        Target value to check.
    type_strict : bool, default True
        Whether strictly check actual value or not.
        For example, if type_strict is True,
        an integer of 1 fails tests. On the contrary
        (if type_strict is False), an integer of 1 passes tests.
    msg : str, optional
        Message to display when assertion failed.

    References
    ----------
    - assert_true and assert_false interfaces document
        - https://simon-ritchie.github.io/apysc/assert_true_and_false.html

    Examples
    --------
    >>> import apysc as ap
    >>> int_val: ap.Int = ap.Int(10)
    >>> boolean: ap.Boolean = int_val == 10
    >>> ap.assert_true(boolean)
    """
    import apysc as ap
    from apysc._string import string_util
    _trace_info(
        interface_label='assert_true', left='true', right=value)
    _, value_str = _get_left_and_right_strs(
        left='_', right=value)

    msg = string_util.escape_str(string=msg)
    expression: str = (
        f'console.assert({value_str} =='
    )
    expression = _add_equal_if_type_strict_setting_is_true(
        expression=expression, type_strict=type_strict)
    expression += f' true, "{msg}");'
    ap.append_js_expression(expression=expression)


@add_debug_info_setting(  # type: ignore[misc]
    module_name=__name__)
def assert_false(
        value: Any, *, type_strict: bool = True, msg: str = '') -> None:
    """
    JavaScript assertion interface for false condition.

    Parameters
    ----------
    value : *
        Target value to check.
    type_strict : bool, default True
        Whether strictly check actual value or not.
        For example, if type_strict is True, an integer
        of 0 fails tests. On the contrary (if type_strict
        is False), an integer of 0 passes tests.
    msg : str, optional
        Message to display when assertion failed.

    References
    ----------
    - assert_true and assert_false interfaces document
        - https://simon-ritchie.github.io/apysc/assert_true_and_false.html

    Examples
    --------
    >>> import apysc as ap
    >>> int_val: ap.Int = ap.Int(10)
    >>> boolean: ap.Boolean = int_val == 11
    >>> ap.assert_false(boolean)
    """
    import apysc as ap
    from apysc._string import string_util
    _trace_info(
        interface_label='assert_false', left='false', right=value)
    _, value_str = _get_left_and_right_strs(
        left='_', right=value)

    msg = string_util.escape_str(string=msg)
    expression: str = (
        f'console.assert({value_str} =='
    )
    expression = _add_equal_if_type_strict_setting_is_true(
        expression=expression, type_strict=type_strict)
    expression += f' false, "{msg}");'
    ap.append_js_expression(expression=expression)


@add_debug_info_setting(  # type: ignore[misc]
    module_name=__name__)
def assert_arrays_equal(
        left: Any, right: Any, *, msg: str = '') -> None:
    """
    JavaScript assertion interface for Array values equal condition.

    Notes
    -----
    This interface is used instead of assert_equal for
    Array class comparison (JavaScript can not compare
    arrays directly, like a Python, for example,
    `[1, 2] === [1, 2]` becomes false).

    Parameters
    ----------
    left : *
        Left-side value to compare.
    right : *
        Right-side value to compare.
    msg : str, optional
        Message to display when assertion failed.

    References
    ----------
    - assert_arrays_equal and assert_arrays_not_equal interfaces document
        - https://simon-ritchie.github.io/apysc/assert_arrays_equal_and_arrays_not_equal.html  # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> arr_1: ap.Array = ap.Array([1, 2, 3])
    >>> arr_2: ap.Array = ap.Array([1, 2, 3])
    >>> ap.assert_arrays_equal(arr_1, arr_2)
    """
    import apysc as ap
    _trace_arrays_or_dicts_assertion_info(
        interface_label='assert_arrays_equal',
        left=left, right=right)

    expression: str = _make_arrays_or_dicts_comparison_expression(
        left=left, right=right, msg=msg, not_condition=False)
    ap.append_js_expression(expression=expression)


@add_debug_info_setting(  # type: ignore[misc]
    module_name=__name__)
def assert_arrays_not_equal(
        left: Any, right: Any, *, msg: str = '') -> None:
    """
    JavaScript assertion interface for Array values not equal condition.

    Notes
    -----
    This interface is used instead of assert_not_equal for
    Array class comparison (JavaScript can not compare arrays
    directly, like a Python, for example,
    `[1, 2] === [1, 2]` becomes false).

    Parameters
    ----------
    left : *
        Left-side value to compare.
    right : *
        Right-side value to compare.
    msg : str, optional
        Message to display when assertion failed.

    References
    ----------
    - assert_arrays_equal and assert_arrays_not_equal interfaces document
        - https://simon-ritchie.github.io/apysc/assert_arrays_equal_and_arrays_not_equal.html  # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> arr_1: ap.Array = ap.Array([1, 2, 3])
    >>> arr_2: ap.Array = ap.Array([4, 5, 6])
    >>> ap.assert_arrays_not_equal(arr_1, arr_2)
    """
    import apysc as ap
    _trace_arrays_or_dicts_assertion_info(
        interface_label='assert_arrays_not_equal',
        left=left, right=right)

    expression: str = _make_arrays_or_dicts_comparison_expression(
        left=left, right=right, msg=msg, not_condition=True)
    ap.append_js_expression(expression=expression)


@add_debug_info_setting(  # type: ignore[misc]
    module_name=__name__)
def assert_dicts_equal(left: Any, right: Any, *, msg: str = '') -> None:
    """
    JavaScript assertion interface for Dictionary values equal
    condition.

    Notes
    -----
    This interface is used instead of assert_equal for
    Dictionary class comparison (JavaScript can not compare
    dictionary (Object) directly, like a Python, for example,
    `{"a": 10} === {"a": 10}` becomes false).

    Parameters
    ----------
    left : *
        Left-side value to compare.
    right : *
        Right-side value to compare.
    msg : str, optional
        Message to display when assertion failed.

    References
    ----------
    - assert_dicts_equal and assert_dicts_not_equal interfaces document
        - https://simon-ritchie.github.io/apysc/assert_dicts_equal_and_dicts_not_equal.html  # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> dict_1: ap.Dictionary = ap.Dictionary({'a': 10})
    >>> dict_2: ap.Dictionary = ap.Dictionary({'a': 10})
    >>> ap.assert_dicts_equal(dict_1, dict_2)
    """
    import apysc as ap
    _trace_arrays_or_dicts_assertion_info(
        interface_label='assert_dicts_equal',
        left=left, right=right)

    expression: str = _make_arrays_or_dicts_comparison_expression(
        left=left, right=right, msg=msg, not_condition=False)
    ap.append_js_expression(expression=expression)


@add_debug_info_setting(  # type: ignore[misc]
    module_name=__name__)
def assert_dicts_not_equal(
        left: Any, right: Any, *, msg: str = '') -> None:
    """
    JavaScript assertion interface for Dictionary values not equal
    condition.

    Notes
    -----
    This interface is used instead of assert_not_equal for
    Dictionary class comparison (JavaScript can not
    compare dictionary (Object) directly, like a Python,
    for example, `{"a": 10} !== {"a": 10}` becomes true).

    Parameters
    ----------
    left : *
        Left-side value to compare.
    right : *
        Right-side value to compare.
    msg : str, optional
        Message to display when assertion failed.

    References
    ----------
    - assert_dicts_equal and assert_dicts_not_equal interfaces document
        - https://simon-ritchie.github.io/apysc/assert_dicts_equal_and_dicts_not_equal.html  # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> dict_1: ap.Dictionary = ap.Dictionary({'a': 10})
    >>> dict_2: ap.Dictionary = ap.Dictionary({'a': 20})
    >>> ap.assert_dicts_not_equal(dict_1, dict_2)
    """
    import apysc as ap
    _trace_arrays_or_dicts_assertion_info(
        interface_label='assert_dicts_not_equal',
        left=left, right=right)

    expression: str = _make_arrays_or_dicts_comparison_expression(
        left=left, right=right, msg=msg, not_condition=True)
    ap.append_js_expression(expression=expression)


@add_debug_info_setting(  # type: ignore[misc]
    module_name=__name__)
def assert_defined(value: Any, *, msg: str = '') -> None:
    """
    JavaScript assertion interface for defined (not undefined)
    value condition.

    Parameters
    ----------
    value : *
        Target value to check.
    msg : str, optional
        Message to display when assertion failed.

    References
    ----------
    - assert_defined and assert_undefined interfaces document
        - https://simon-ritchie.github.io/apysc/assert_defined_and_undefined.html  # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> int_val: ap.Int = ap.Int(10)
    >>> ap.assert_defined(int_val)
    """
    import apysc as ap
    from apysc._string import string_util
    _trace_info(
        interface_label='assert_defined', left='other than undefined',
        right=value)
    _, value_str = _get_left_and_right_strs(
        left='_', right=value)

    msg = string_util.escape_str(string=msg)
    expression: str = (
        f'console.assert(!_.isUndefined({value_str}), "{msg}");'
    )
    ap.append_js_expression(expression=expression)


@add_debug_info_setting(  # type: ignore[misc]
    module_name=__name__)
def assert_undefined(value: Any, *, msg: str = '') -> None:
    """
    JavaScript assertion interface for undefined value condition.

    Parameters
    ----------
    value : *
        Target value to check.
    msg : str, optional
        Message to display when assertion failed.

    References
    ----------
    - assert_defined and assert_undefined interfaces document
        - https://simon-ritchie.github.io/apysc/assert_defined_and_undefined.html  # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> int_val: ap.Int = ap.Int(10)
    >>> ap.append_js_expression(
    ...     expression=f'{int_val.variable_name} = undefined;')
    >>> ap.assert_undefined(int_val)
    """
    import apysc as ap
    from apysc._string import string_util
    _trace_info(
        interface_label='assert_undefined', left='undefined',
        right=value)
    _, value_str = _get_left_and_right_strs(
        left='_', right=value)

    msg = string_util.escape_str(string=msg)
    expression: str = (
        f'console.assert(_.isUndefined({value_str}), "{msg}");'
    )
    ap.append_js_expression(expression=expression)


@add_debug_info_setting(  # type: ignore[misc]
    module_name=__name__)
def _make_arrays_or_dicts_comparison_expression(
        *, left: Any, right: Any, msg: str,
        not_condition: bool) -> str:
    """
    Make arrays or dicts comparison (assert_arrays_equal,
    assert_arrays_not_equal, assert_dicts_equal, or
    assert_dicts_not_equal) expression string.

    Parameters
    ----------
    left : *
        Left-side value to compare.
    right : *
        Right-side value to compare.
    msg : str, optional
        Message to display when assertion failed.
    not_condition : bool
        Boolean value whether this expression is not condition
        (assert_arrays_not_equal) or not.

    Returns
    -------
    expression : str
        A result expression string.
    """
    from apysc._string import string_util
    from apysc._type import value_util
    left_exp_str: str = value_util.get_value_str_for_expression(
        value=left)
    right_exp_str: str = value_util.get_value_str_for_expression(
        value=right)
    msg = string_util.escape_str(string=msg)
    if not_condition:
        not_condition_str: str = '!'
    else:
        not_condition_str = ''
    expression: str = (
        f'console.assert({not_condition_str}_.isEqual({left_exp_str}, '
        f'{right_exp_str}), "{msg}");'
    )
    return expression


@add_debug_info_setting(  # type: ignore[misc]
    module_name=__name__)
def _trace_arrays_or_dicts_assertion_info(
        *, interface_label: str, left: Any, right: Any) -> None:
    """
    Append arrays or dictionaries values' information
    trace expression.

    Parameters
    ----------
    interface_label : str
        Target assertion interface label, e.g., 'assert_arrays_equal'.
    left : *
        Left-side value to compare.
    right : *
        Right-side value to compare.
    """
    import apysc as ap
    from apysc._type import value_util
    left_exp_str: str = value_util.get_value_str_for_expression(
        value=left)
    if isinstance(left, dict):
        left_exp_str = left_exp_str.replace('"', '')
    right_exp_str: str = value_util.get_value_str_for_expression(
        value=right)
    if isinstance(right, dict):
        right_exp_str = right_exp_str.replace('"', '')
    if isinstance(left, (ap.Array, ap.Dictionary)):
        value_str: str = value_util.get_value_str_for_expression(
            value=left.value)
        value_str = value_str.replace('"', '')
        left_info_str: str = f'{left_exp_str} ({value_str})'
    else:
        left_info_str = left_exp_str
    right_info_str = right_exp_str
    _trace_info(
        interface_label=interface_label,
        left=left_info_str,
        right=right_info_str)


def _value_type_is_array(*, value: Any) -> bool:
    """
    Get a boolean value whether the specified value is
    Array type or not.

    Parameters
    ----------
    value : *
        Target value to check.

    Returns
    -------
    result : bool
        If the value type is Array, this interface returns True.
    """
    import apysc as ap
    if isinstance(value, ap.Array):
        return True
    return False


def _value_type_is_dict(*, value: Any) -> bool:
    """
    Get a boolean value whether the specified value is
    Dictionary type or not.

    Parameters
    ----------
    value : *
        Target value to check.

    Returns
    -------
    result : bool
        If the value type is Dictionary, this interface returns True.
    """
    from apysc._type.dictionary_structure import DictionaryStructure
    if isinstance(value, DictionaryStructure):
        return True
    return False


def _add_equal_if_type_strict_setting_is_true(
        *, expression: str, type_strict: bool) -> str:
    """
    Add single equal character to expression if type_string setting
    is True.

    Parameters
    ----------
    expression : str
        Expression to be added.
    type_strict: bool
        Type strict setting value.

    Returns
    -------
    expression : str
        If the type_strict setting is True value,
        this interface appends a single equal character to tail.
    """
    if not type_strict:
        return expression
    expression += '='
    return expression


def _get_left_and_right_strs(
        *, left: Any, right: Any) -> Tuple[str, str]:
    """
    Get left and right value strings from specified values.

    Parameters
    ----------
    left : *
        Left-side value to compare.
    right : *
        Right-side value to compare.

    Returns
    -------
    left_str : str
        Left-side value's string. If a value is a string,
        this interface wraps this left-side value by
        double quotation.
    right_str : str
        Right-side value's string. If a value is a string,
        this interface wraps this right-side value by
        double quotation.
    """
    from apysc._type import value_util
    left_str: str = value_util.get_value_str_for_expression(
        value=left)
    right_str: str = value_util.get_value_str_for_expression(value=right)
    return left_str, right_str


@add_debug_info_setting(  # type: ignore[misc]
    module_name=__name__)
def _trace_info(*, interface_label: str, left: Any, right: Any) -> None:
    """
    Append trace expression of specified values.

    Parameters
    ----------
    interface_label : str
        Target assertion interface label, e.g., 'assert_equal'.
    left : *
        Left-side value to compare.
    right : *
        Right-side value to compare.
    """
    import apysc as ap
    from apysc._type.variable_name_interface import VariableNameInterface
    info: str = f'[{interface_label}]'
    if isinstance(left, VariableNameInterface):
        info += f'\nLeft-side variable name: {left.variable_name}'
    if isinstance(right, VariableNameInterface):
        info += f'\nRight-side variable name: {right.variable_name}'
    ap.trace(info, '\nLeft value:', left, 'right value:', right)



"""This module is for the translation mapping data of the
following document:

Document file: array_extend_and_concat.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Array class extend and concat interfaces':
    '',

    'This page explains the `Array` class `extend` and `concat` method interfaces.':  # noqa
    '',

    '## What interfaces are these?':
    '',

    'The `extend` and `concat` method interfaces are the two arrays\' concatenation interfaces.\n\nThe `extend` method updates an original array in place and returns the `None`. The `concat` method returns the concatenated array, and an original one is not updated.':  # noqa
    '',

    '## Basic usage':
    '',

    'The `extend` and `concat` methods require other iterable objects, like the `list`\\, `tuple`\\, or `Array` value at the first argument, as follows:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 2])\narr.extend([3, 4])\nassert arr == [1, 2, 3, 4]\n\nother_arr: ap.Array[int] = arr.concat([5, 6])\nassert other_arr == [1, 2, 3, 4, 5, 6]\nassert arr == [1, 2, 3, 4]\n```':  # noqa
    '',

    '':
    '',

    '## extend API':
    '',

    '<!-- Docstring: apysc._type.array.Array.extend -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `extend(self, other_arr:Union[List[~T], tuple, _ForwardRef(\'Array\')]) -> None`<hr>\n\n**[Interface summary]** Concatenate argument array to this one. This interface positions the argument array\'s values after this array values. This method is similar to the concat method. Still, there is a difference in whether updating the same variable (extend) or returned as a different variable (concat).<hr>\n\n**[Parameters]**\n\n- `other_arr`: Array or list or tuple\n  - Other array-like values to concatenate.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr.extend([4, 5, 6])\n>>> arr\nArray([1, 2, 3, 4, 5, 6])\n```':  # noqa
    '',

    '':
    '',

    '## concat API':
    '',

    '<!-- Docstring: apysc._type.array.Array.concat -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `concat(self, other_arr:Union[List[~T], tuple, _ForwardRef(\'Array\')]) -> \'Array\'`<hr>\n\n**[Interface summary]** Concatenate argument array to this one. This interface positions the argument array\'s values after this array values. This method is similar to extend method, but there is a difference in whether updating the same variable (extend) or returned as a different variable (concat).<hr>\n\n**[Parameters]**\n\n- `other_arr`: Array or list or tuple\n  - Other array-like values to concatenate.\n\n<hr>\n\n**[Returns]**\n\n- `concatenated`: Array\n  - Concatenated array value.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr = arr.concat([4, 5, 6])\n>>> arr\nArray([1, 2, 3, 4, 5, 6])\n```':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: dictionary_get.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Dictionary get interface':
    '',

    'This page explains the `Dictionary` class `get` method interface.':
    '',

    '## What interface is this?':
    '',

    'The `get` method returns the specified key\'s value. If that key does not exist in the dictionary, it returns the default value (not raising an exception).':  # noqa
    '',

    '## Basic usage':
    '',

    'The `get` method requires the first argument, `key` (dictionary key). The second argument of the `default` is optional, and if not provided, it returns the `None` value.':  # noqa
    '',

    '```py\n# runnable\nfrom typing import Any, Optional\n\nimport apysc as ap\n\ndict_val: ap.Dictionary = ap.Dictionary({\'a\': 10})\ngot_val_1: int = dict_val.get(key=\'a\', default=0)\nassert got_val_1 == 10\n\ngot_val_2: int = dict_val.get(key=\'b\', default=0)\nassert got_val_2 == 0\n\ngot_val_3: Optional[Any] = dict_val.get(key=\'b\')\nassert got_val_3 is None\n```':  # noqa
    '',

    '':
    '',

    '## get API':
    '',

    '<!-- Docstring: apysc._type.dictionary.Dictionary.get -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `get(self, key:Union[~_K, apysc._type.expression_string.ExpressionString], *, default:~DefaultType=None) -> ~DefaultType`<hr>\n\n**[Interface summary]** Get a specified key dictionary value. If this dictionary hasn\'t a specified key, this interface returns a default value.<hr>\n\n**[Parameters]**\n\n- `key`: _K\n  - Target key.\n- `default`: DefaultType or None, optional\n  - Any default value.\n\n<hr>\n\n**[Returns]**\n\n- `result_value`: Any\n  - Extracted value or a default value.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> from typing import Optional\n>>> import apysc as ap\n>>> dictionary: ap.Dictionary = ap.Dictionary({\'a\': 10})\n>>> value_1: Optional[int] = dictionary.get(\'a\')\n>>> value_1\n10\n\n>>> value_2: Optional[int] = dictionary.get(\'b\')\n>>> print(value_2)\nNone\n\n>>> value_3: int = dictionary.get(\'c\', default=0)\n>>> value_3\n0\n```':  # noqa
    '',

}



"""Definition of js function expressions (e.g., common helper function).
"""

import inspect
import sys
from types import ModuleType
from typing import Any
from typing import List
from typing import Tuple

from typing_extensions import Final

# Function for deepcopy.
FUNC_COPY: Final[str] = """function cpy(any_obj) {
  return JSON.parse(JSON.stringify(any_obj));
}"""

# Function to get a specified object's total x-coordinate (including
# parents coordinates).
FUNC_GET_TOTAL_X: Final[str] = """function get_total_x(any_obj) {
    var total_x = 0;
    total_x += any_obj.attr("x");
    var parent = any_obj.parent();
    while (!_.isUndefined($(parent).attr("x"))) {
        total_x += parent.attr("x");
        parent = parent.parent();
    }
    return total_x;
}"""

# Function to get a specified object's total y-coordinate (including
# parents coordinates).
FUNC_GET_TOTAL_Y: Final[str] = """function get_total_y(any_obj) {
    var total_y = 0;
    total_y += any_obj.attr("y");
    var parent = any_obj.parent();
    while (!_.isUndefined($(parent).attr("y"))) {
        total_y += parent.attr("y");
        parent = parent.parent();
    }
    return total_y;
}"""


def get_js_functions() -> List[str]:
    """
    Get js function expressions defined in this module.

    Returns
    -------
    js_function_strs : list of str
        js function expressions defined in this module.
        This interface returns the list of string constants
        with the `FUNC_` name prefix.
    """
    this_module: ModuleType = sys.modules[__name__]
    members: List[Tuple[str, Any]] = inspect.getmembers(this_module)
    js_function_strs: List[str] = []
    for member_name, member_val in members:
        if not isinstance(member_val, str):
            continue
        if not member_name.startswith('FUNC_'):
            continue
        js_function_strs.append(member_val)
    return js_function_strs



"""This module is for the translation mapping data of the
following document:

Document file: display_object_visible.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Display object visible interface':
    '',

    'This page explains the `DisplayObject` class `visible` property interface.':  # noqa
    '',

    '## What interface is this?':
    '',

    'The `visible` property interface will change the `DisplayObject` visible / invisible state.':  # noqa
    '',

    '## Basic usage':
    '',

    'The `visible` property accepts a `Boolean` value. If you set the True value, a `DisplayObject` instance becomes visible (default). Conversely, if you set the False value, a `DisplayObject` instance becomes invisible.\n\nThe following example switches the visible values when you click the rectangle. For example, suppose you click the left rectangle (the rectangle_1). In that case, the left rectangle becomes invisible, and the right rectangle (rectangle_2) becomes visible.':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_rectangle_1_click(\n        e: ap.MouseEvent[ap.Rectangle], options: _RectOptions) -> None:\n    """\n    The handler that the first rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle_1: ap.Rectangle = e.this\n    rectangle_2: ap.Rectangle = options[\'rectangle\']\n    rectangle_1.visible = ap.Boolean(False)\n    rectangle_2.visible = ap.Boolean(True)\n\n\ndef on_rectangle_2_click(\n        e: ap.MouseEvent[ap.Rectangle], options: _RectOptions) -> None:\n    """\n    The handler that the second rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle_1: ap.Rectangle = options[\'rectangle\']\n    rectangle_2: ap.Rectangle = e.this\n    rectangle_1.visible = ap.Boolean(True)\n    rectangle_2.visible = ap.Boolean(False)\n\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=250,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\n\nsprite.graphics.begin_fill(color=\'#f0a\')\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n    x=150, y=50, width=50, height=50)\nrectangle_2.visible = ap.Boolean(False)\n\noptions: _RectOptions = {\'rectangle\': rectangle_2}\nrectangle_1.click(\n    on_rectangle_1_click, options=options)\noptions = {\'rectangle\': rectangle_1}\nrectangle_2.click(\n    on_rectangle_2_click, options=options)\n\nap.save_overall_html(\n    dest_dir_path=\'display_object_visible_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/display_object_visible_basic_usage/index.html" width="250" height="150"></iframe>':  # noqa
    '',

    '## visible property API':
    '',

    '<!-- Docstring: apysc._display.visible_interface.VisibleInterface.visible -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get a visibility value of this instance.<hr>\n\n**[Returns]**\n\n- `result`: Boolean\n  - If this instance is visible, this interface returns True.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> rectangle.visible = ap.Boolean(False)\n>>> rectangle.visible\nBoolean(False)\n```':  # noqa
    '',

}



"""Common string utilities.

Mainly following interfaces and defined.

- escape_str
    - Escape special characters (e.g., line breaks).
- escape_double_quotation
    - Escape double quotations.
- wrap_by_double_quotation_if_value_is_string
    - Wrap specified value by double quotation if a value
    is a string.
- substitute_file_by_pattern
    - Substitute text file by regular expression pattern.
- replace_double_spaces_to_single_space
    - Replace double spaces with a single space.
"""

import re
from typing import Any
from typing import TypeVar


def escape_str(*, string: str) -> str:
    """
    Escape special characters (e.g. line breaks of `\n`).

    Parameters
    ----------
    string : str
        String to escape.

    Returns
    -------
    string : str
        Escaped string.
    """
    string = repr(string)[1:-1]
    return string


def escape_double_quotation(*, string: str) -> str:
    """
    Escape double quotations.

    Parameters
    ----------
    string : str
        String to escape.

    Returns
    -------
    string : str
        Escaped string.
    """
    string = string.replace('"', '\\"')
    return string


T = TypeVar('T')


def wrap_by_double_quotation_if_value_is_string(*, value: T) -> T:
    """
    Wrap specified value by double quotation if a
    value is a string.

    Parameters
    ----------
    value : *
        Any value to wrap.

    Returns
    -------
    value : *
        Wrapped value. If a not-string value is specified,
        return that value immediately.
    """
    if not isinstance(value, str):
        return value
    value = f'"{value}"'  # type: ignore
    return value  # type: ignore


def substitute_file_by_pattern(
        *, file_path: str, pattern: str, repl: str,
        flags: Any) -> None:
    """
    Substitute text file by regular expression pattern.

    Parameters
    ----------
    file_path : str
        Target file path.
    pattern : str
        Regular expression pattern.
    repl : str
        String that replace matched pattern one.
    flags : Any
        Regular expression flags.
    """
    from apysc._file import file_util
    string: str = file_util.read_txt(file_path=file_path)
    string = re.sub(
        pattern=pattern,
        repl=repl,
        string=string,
        flags=flags)
    file_util.save_plain_txt(txt=string, file_path=file_path)


def replace_double_spaces_to_single_space(*, string: str) -> str:
    """
    Replace double spaces with a single space.

    Parameters
    ----------
    string : str
        Target string to replace.

    Returns
    -------
    string : str
        Replaced string.
    """
    while '  ' in string:
        string = string.replace('  ', ' ')
    return string



"""Line joints enum's definitions.
"""

from enum import Enum


class LineJoints(Enum):
    """
    The class for the line joints' style type definitions.

    Attributes
    ----------
    MITER
        Default style type. Line vertices will be miter joint.
    ROUND
        Line vertices will be rounded.
    BEVEL
        Line vertices will be bevel joint (bevel vertices).

    References
    ----------
    - Graphics line_style interface document
        - https://simon-ritchie.github.io/apysc/graphics_line_style.html
    """
    MITER = 'miter'
    ROUND = 'round'
    BEVEL = 'bevel'



"""This module is for the translation mapping data of the
following document:

Document file: event_prevent_default_and_stop_propagation.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Event class prevent_default and stop_propagation interfaces':
    '',

    'This page explains the `Event` class `prevent_default` and `stop_propagation` method interfaces.':  # noqa
    '',

    '## What interfaces are these?':
    '',

    'The `prevent_default` method interface appends the calling expression of the JavaScript `preventDefault` method. This interface prevents the browser default behavior of any event.\n\nThe `stop_propagation` method interface stops an event\'s propagation; for example, the triggered child event does not propagate to a parent event (it ignores the parent event).':  # noqa
    '',

    '## Basic usage of the prevent_default interface':
    '',

    'The `Event` instance and its subclass instance have the `prevent_default` method. The `prevent_default` method requires no arguments, as follows:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(\n        e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.prevent_default()\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(\n    dest_dir_path=\'event_prevent_default_basic_usage/\')\n```':  # noqa
    '',

    '':
    '',

    '## Basic usage of the stop_propagation interface':
    '',

    'The `Event` instance and its subclass instance have the `stop_propagation` method. The `stop_propagation` method, like the `prevent_default` one, requires no arguments.\n\nThe following example binds the click event to the sprite and rectangle instances. The rectangle (child) click handler calls the `stop_propagation` method, so the sprite (parent) doesn\'t call the click handler:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_rectangle_click(\n        e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.stop_propagation()\n    ap.trace(\'The rectangle is clicked!\')\n\n\ndef on_sprite_click(\n        e: ap.MouseEvent[ap.Sprite], options: dict) -> None:\n    """\n    The handler that the sprite calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.trace(\'The sprite is clicked!\')\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.click(on_sprite_click)\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle.click(on_rectangle_click)\n\nap.save_overall_html(\n    dest_dir_path=\'event_stop_propagation_basic_usage/\')\n```':  # noqa
    '',

    'If you click the following rectangle, the only message of `The rectangle is clicked!` is displayed browser console (please press the F12 key). Also, the sprite console message is not displayed.\n\n<iframe src="static/event_stop_propagation_basic_usage/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## prevent_default API':
    '',

    '<!-- Docstring: apysc._event.prevent_default_interface.PreventDefaultInterface.prevent_default -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `prevent_default(self) -> None`<hr>\n\n**[Interface summary]** Prevent event\'s default behavior.<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_click(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     e.prevent_default()\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String(\'#f0a\')\n...     rectangle.unbind_mouseup_all()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.click(on_click)\n```':  # noqa
    '',

    '':
    '',

    '## stop_propagation API':
    '',

    '<!-- Docstring: apysc._event.stop_propagation_interface.StopPropagationInterface.stop_propagation -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `stop_propagation(self) -> None`<hr>\n\n**[Interface summary]** Stop event propagation.<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_click(e: ap.MouseEvent, options: dict) -> None:\n...     e.stop_propagation()\n...     ap.trace(\'Clicked!\')\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = sprite.click(on_click)\n>>> _ = rectangle.click(on_click)\n```':  # noqa
    '',

}



"""Module related common utilities.

Mainly following interfaces are defined:

- get_module_paths_recursively
    - Get all module paths under the specified directory.
- save_tmp_module
    - Save a temporary Python module.
- save_tmp_module_and_run_script
    - Save a temporary Python module and run that script.
- read_target_path_module
    - Read a module of the specified module path.
- read_module_or_class_from_package_path
    - Read a specified package path module or class.
"""

import importlib
import os
import subprocess as sp
from datetime import datetime
from random import randint
from types import ModuleType
from typing import Any
from typing import List
from typing import Optional


def get_module_paths_recursively(
        dir_path: str = './',
        *,
        module_paths: Optional[List[str]] = None) -> List[str]:
    """
    Get all module paths under the specified directory.

    Parameters
    ----------
    dir_path : str
        Directory path to search modules.
    module_paths : list of str or None
        Current Python module paths (only used by recursive
        function calls).

    Returns
    -------
    module_paths : list of str
        Python module paths. This interface does not
        include the `__init__.py` modules.
    """
    if module_paths is None:
        module_paths = []
    file_or_dir_names: List[str] = os.listdir(dir_path)
    for file_or_dir_name in file_or_dir_names:
        file_or_dir_path: str = os.path.join(dir_path, file_or_dir_name)
        if os.path.isdir(file_or_dir_path):
            module_paths = get_module_paths_recursively(
                dir_path=file_or_dir_path, module_paths=module_paths)
            continue
        if not file_or_dir_path.endswith('.py'):
            continue
        if file_or_dir_name == '__init__.py':
            continue
        module_paths.append(file_or_dir_path)
    return module_paths


def save_tmp_module(*, script: str) -> str:
    """
    Save a temporary Python module.

    Parameters
    ----------
    script : str
        Python script string.

    Returns
    -------
    saved_module_path : str
        Saved temporary module path.
    """
    random_int: int = randint(1_000_000_000, 10_000_000_000)
    saved_module_path: str = (
        f'./tmp_{datetime.now().timestamp()}_{random_int}.py')
    with open(saved_module_path, 'w') as f:
        f.write(script)
    return saved_module_path


def save_tmp_module_and_run_script(*, script: str) -> str:
    """
    Save a temporary Python module and run that script.

    Parameters
    ----------
    script : str
        Python script string.

    Returns
    -------
    stdout : str
        Result stdout string.
    """
    tmp_mod_path: str = save_tmp_module(script=script)
    process: sp.CompletedProcess = sp.run(
        f'python {tmp_mod_path}', shell=True,
        stdout=sp.PIPE, stderr=sp.STDOUT)
    stdout: str = process.stdout.decode('utf-8')
    os.remove(tmp_mod_path)
    return stdout


def read_target_path_module(module_path: str) -> ModuleType:
    """
    Read a module of the specified module path.

    Parameters
    ----------
    module_path : str
        Target module path.

    Returns
    -------
    module : ModuleType
        Read module.
    """
    if module_path.startswith('./'):
        module_path = module_path.replace('./', '', 1)
    module_path = module_path.rsplit('.py', maxsplit=1)[0]
    module_path = module_path.replace('/', '.')
    module: ModuleType = importlib.import_module(name=module_path)
    return module


def read_module_or_class_from_package_path(
        module_or_class_package_path: str) -> Any:
    """
    Read a specified package path module or class.

    Parameters
    ----------
    module_or_class_package_path : str
        Target package module or class path.

    Returns
    -------
    module_or_class : ModuleType or Type
        Read module or class.
    """
    try:
        module: ModuleType = importlib.import_module(
            module_or_class_package_path)
    except Exception:
        splitted: List[str] = module_or_class_package_path.rsplit(
            '.', maxsplit=1)
        module_or_class_package_path = splitted[0]
        class_name: str = splitted[1]
        module = importlib.import_module(
            module_or_class_package_path)
        return getattr(module, class_name)
    return module



"""This module is for the translation mapping data of the
following document:

Document file: else.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Else':
    '',

    'This page explains the `Else` class.\n\nBefore reading on, maybe it is helpful to read the following page (the apysc uses the `Else` class for the same reason of each apysc data type):\n\n- [Why the apysc library doesn\'t use the Python built-in data type](why_apysc_doesnt_use_python_builtin_data_type.md)':  # noqa
    '',

    '## What is the Else class?':
    '',

    'The `Else` class is the apysc branch instruction class. It behaves like the Python built-in `else` keyword.':  # noqa
    '',

    '## Basic usage':
    '',

    'The `Else` requires using the `with` statement. The `Else` class statement is only acceptable to implement right after the `If` or `Elif` classes statement.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\ncondition: ap.Boolean = ap.Boolean(False)\nint_1: ap.Int = ap.Int(10)\n\nwith ap.If(condition):\n    int_1 += 10\nwith ap.Else():\n    int_1 += 20\n```':  # noqa
    '',

    '':
    '',

    '## Notes':
    '',

    'if you insert the code between the `If` (or `Elif`) and `Else` statements, it raises an exception:':  # noqa
    '',

    '```py\nimport apysc as ap\n\ncondition: ap.Boolean = ap.Boolean(False)\nint_1: ap.Int = ap.Int(10)\n\nwith ap.If(condition):\n    int_1 += 10\n# If there is a code implementation between the `If` and `Else`, then\n# exceptions will be raised.\nint_2: ap.Int = ap.Int(20)\nwith ap.Else():\n    int_1 += 20\n```':  # noqa
    '',

    '':
    '',

    '```\nValueError: Else interface can only use right after If or Elif interfaces.\n```':  # noqa
    '',

    '':
    '',

    '## See also':
    '',

    '- [If class](if.md)\n- [Elif class](elif.md)\n- [Each branch instruction class\'s scope variables reverting setting](branch_instruction_variables_reverting_setting.md)':  # noqa
    '',

    '## Else constructor API':
    '',

    '<!-- Docstring: apysc._branch._else.Else.__init__ -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `__init__(self, *, locals_:Union[Dict[str, Any], NoneType]=None, globals_:Union[Dict[str, Any], NoneType]=None) -> None`<hr>\n\n**[Interface summary]** A class to append else branch instruction expression.<hr>\n\n**[Parameters]**\n\n- `locals_`: dict or None, default None\n  - Current scope\'s local variables. Set locals() value to this argument. If specified, this interface reverts all local scope VariableNameInterface variables (like Int, Sprite) at the end of an `Else` scope. This setting is useful when you don\'t want to update each variable by implementing the `Else` scope.\n- `globals_`: dict or None, default None\n  - Current scope\'s global variables. Set globals() value to this argument. This setting works the same way as the locals_ argument.\n\n<hr>\n\n**[Notes]**\n\n ・You can only use this class immediately after the `If` or `Elif` statement.<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> int_val: ap.Int = ap.Int(10)\n>>> with ap.If(int_val >= 11):\n...     ap.trace(\'Value is greater than equal 11.\')\n>>> with ap.Else():\n...     ap.trace(\'Value is less than 11.\')\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [Each branch instruction class\'s scope variables reverting setting](https://simon-ritchie.github.io/apysc/branch_instruction_variables_reverting_setting.html)':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: graphics.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Graphics':
    '',

    'This page explains the `Graphics` class.':
    '',

    '## What is Graphics?':
    '',

    'The `Graphics` is the class to handle each vector graphics interface. This interface has the draw rectangle interface, draw line interface, or something else.\n\nThe `Sprite` or other `DisplayObject` instances instantiate this class instance.':  # noqa
    '',

    '## Call interfaces from sprite instance':
    '',

    'Sprite (object container) instance has the `graphics` attribute to call each drawing interface with this attribute.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=250,\n    stage_height=180,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\n\n# Draw the white border and cyan color rectangle.\nsprite.graphics.line_style(color=\'#fff\', thickness=5)\nsprite.graphics.begin_fill(color=\'#0af\')\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\n\n# Draw the magenta color polyline.\nsprite.graphics.begin_fill(color=\'\')\nsprite.graphics.line_style(color=\'#f0a\', thickness=5)\nsprite.graphics.move_to(x=150, y=50)\nsprite.graphics.line_to(x=200, y=50)\nsprite.graphics.line_to(x=150, y=100)\nsprite.graphics.line_to(x=200, y=100)\n\n# Draw the dashed line.\nsprite.graphics.draw_dashed_line(\n    x_start=50, y_start=130, x_end=200, y_end=130,\n    dash_size=10, space_size=5)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_call_interfaces_from_sprite_instance/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_call_interfaces_from_sprite_instance/index.html" width="250" height="180"></iframe>':  # noqa
    '',

    '## Return values':
    '',

    'Each interface returns created graphic instances (e.g., `Rectangle`\\, `Polyline`\\, and so on). These instances have the basic `DisplayObject` attributes and methods, like x, y, fill_alpha, visible, or something else.\n\nFor example, you can set an event and coordinate\'s updating to these instances, as follows:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=200,\n    stage_height=200,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\n\n\ndef on_rectangle_click(\n        e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n\n    # Update the coordinates, fill alpha, and fill color.\n    rectangle.x = ap.Int(100)\n    rectangle.y = ap.Int(100)\n    rectangle.fill_alpha = ap.Number(0.5)\n    rectangle.fill_color = ap.String(\'#f0a\')\n\n\n# drew_rect interface will return Rectangle instance.\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\n\n# Bind click event to the rectangle.\nrectangle.click(on_rectangle_click)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_return_values/\')\n```':  # noqa
    '',

    'If you click the following rectangle, that rectangle changes x and y coordinates, fill color, and alpha (opacity) values.\n\n<iframe src="static/graphics_return_values/index.html" width="200" height="200"></iframe>':  # noqa
    '',

    '## See also':
    '',

    '- [Graphics class begin fill interface](graphics_begin_fill.md)\n- [Graphics class line style interface](graphics_line_style.md)\n- [Graphics class draw rect interface](graphics_draw_rect.md)\n- [Graphics class draw round rect interface](graphics_draw_round_rect.md)\n- [Graphics class draw circle interface](graphics_draw_circle.md)\n- [Graphics class draw ellipse interface](graphics_draw_ellipse.md)\n- [Graphics class move to and line to interfaces](graphics_move_to_and_line_to.md)\n- [Graphics class draw line interface](graphics_draw_line.md)\n- [Graphics class draw dotted line interface](graphics_draw_dotted_line.md)\n- [Graphics class draw dashed line interface](graphics_draw_dashed_line.md)\n- [Graphics class draw round dotted line interface](graphics_draw_round_dotted_line.md)\n- [Graphics class draw dash dotted line interface](graphics_draw_dash_dotted_line.md)\n- [Graphics class draw polygon interface](graphics_draw_polygon.md)':  # noqa
    '',

}



"""Class implementation of integer.
"""

from typing import Union

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.number_value_interface import NumberValueInterface


class Int(NumberValueInterface[int, 'Int']):
    """
    Integer class for the apysc library.

    References
    ----------
    - Int and Number document
        - https://simon-ritchie.github.io/apysc/int_and_number.html
    - Int and Number common arithmetic operations document
        - https://simon-ritchie.github.io/apysc/int_and_number_arithmetic_operations.html  # noqa
    - Int and Number common comparison operations document
        - https://simon-ritchie.github.io/apysc/int_and_number_comparison_operations.html  # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> int_val: ap.Int = ap.Int(10)
    >>> int_val
    Int(10)

    >>> int_val == 10
    Boolean(True)

    >>> int_val == ap.Int(10)
    Boolean(True)

    >>> int_val >= 10
    Boolean(True)

    >>> int_val += 10
    >>> int_val
    Int(20)

    >>> int_val = ap.Int(10.5)
    >>> int_val
    Int(10)
    """

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='Int')
    def __init__(
            self, value: Union[int, float, NumberValueInterface]) -> None:
        """
        Integer class for apysc library.

        Parameters
        ----------
        value : int or float or Int or Number
            Initial integer value. If the `float` or `Number`
            value is specified, this class casts it to an integer.

        References
        ----------
        - Int and Number document
            - https://simon-ritchie.github.io/apysc/int_and_number.html
        - Int and Number common arithmetic operations document
            - https://simon-ritchie.github.io/apysc/int_and_number_arithmetic_operations.html  # noqa
        - Int and Number common comparison operations document
            - https://simon-ritchie.github.io/apysc/int_and_number_comparison_operations.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> int_val: ap.Int = ap.Int(10)
        >>> int_val
        Int(10)

        >>> int_val == 10
        Boolean(True)

        >>> int_val == ap.Int(10)
        Boolean(True)

        >>> int_val >= 10
        Boolean(True)

        >>> int_val += 10
        >>> int_val
        Int(20)

        >>> int_val = ap.Int(10.5)
        >>> int_val
        Int(10)
        """
        from apysc._converter import cast
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        from apysc._expression.event_handler_scope import \
            TemporaryNotHandlerScope
        from apysc._type import type_util
        with TemporaryNotHandlerScope():
            is_number_specified: bool = type_util.is_number(
                value=value)
            TYPE_NAME: str = var_names.INT
            self.variable_name = expression_variables_util.\
                get_next_variable_name(type_name=TYPE_NAME)
            super(Int, self).__init__(value=value, type_name=TYPE_NAME)
            self._value = cast.to_int_from_float(int_or_float=self.value)
            self.append_constructor_expression()
            self._append_cast_expression(
                is_number_specified=is_number_specified)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='Int')
    def _append_cast_expression(
            self, *, is_number_specified: bool) -> None:
        """
        Append integer cast (parseInt) expression.

        Parameters
        ----------
        is_number_specified : bool
            Boolean value whether a specified value is Number
            instance or not.
        """
        import apysc as ap
        if not is_number_specified:
            return
        expression: str = (
            f'{self.variable_name} = parseInt({self.variable_name}, 10);'
        )
        ap.append_js_expression(expression=expression)

    def _set_value_and_skip_expression_appending(
            self, *,
            value: Union[int, float, NumberValueInterface]) -> None:
        """
        Update value attribute and skip expression appending.

        Parameters
        ----------
        value : int or float or Int or Number
            Any number value to set. This interface casts that
            value to an integer if float or number value is specified.
        """
        from apysc._converter import cast
        from apysc._validation import number_validation
        number_validation.validate_num(num=value)  # type: ignore
        if isinstance(value, NumberValueInterface):
            value._value = cast.to_int_from_float(int_or_float=value._value)
            value_: Union[int, float, NumberValueInterface] = value._value
        else:
            value = cast.to_int_from_float(int_or_float=value)
            value_ = value
        self._value = value_  # type: ignore

    def __repr__(self) -> str:
        """
        Get a representation string of this instance.

        Returns
        -------
        repr_str : str
            Representation string of this instance.
        """
        if not hasattr(self, '_value'):
            repr_str: str = 'Int(0)'
        else:
            repr_str = f'Int({self._value})'
        return repr_str



"""This module is for the translation mapping data of the
following document:

Document file: draw_interfaces_abstract.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Draw interfaces abstract':
    '',

    'This page would explain the drawing interfaces abstract.':
    '',

    '## What apysc can do in its drawing interfaces':
    '',

    '- You can set the fill-color, fill-alpha (opacity), line-color, line-alpha, line-thickness values and draw each SVG graphic with these.\n- Supported graphics: The rectangle, circle, ellipse, polygon, line, polyline, and path.':  # noqa
    '',

    '## Fill settings':
    '',

    'The `begin_fill` interface sets the fill-color and fill-alpha (opacity).':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\', alpha=0.5)\nsprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nap.save_overall_html(\n    dest_dir_path=\'draw_interfaces_abstract_begin_fill/\')\n```':  # noqa
    '',

    '<iframe src="static/draw_interfaces_abstract_begin_fill/index.html" width="150" height="150"></iframe>\n\nFor more details, please see the [Graphics class begin fill interface](graphics_begin_fill.md).':  # noqa
    '',

    '## Line style settings':
    '',

    'The `line_style` interface sets the line-color, line-alpha (opacity), line-thickness.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.line_style(color=\'#fff\', thickness=5, alpha=0.5)\nsprite.graphics.draw_line(x_start=50, y_start=50, x_end=150, y_end=50)\n\nap.save_overall_html(\n    dest_dir_path=\'draw_interfaces_abstract_line_style/\')\n```':  # noqa
    '',

    '<iframe src="static/draw_interfaces_abstract_line_style/index.html" width="200" height="100"></iframe>\n\nFor more details, please see the [Graphics class line style interface](graphics_line_style.md).':  # noqa
    '',

    '## Each drawing interface':
    '',

    'Each drawing interface has the `draw_` prefix draw SVG graphics (e.g., draw_rect, draw_circle).':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=250,\n    stage_height=150,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nsprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nsprite.graphics.draw_circle(x=175, y=75, radius=25)\n\nap.save_overall_html(\n    dest_dir_path=\'draw_interfaces_abstract_each_drawing_interface/\')\n```':  # noqa
    '',

    '<iframe src="static/draw_interfaces_abstract_each_drawing_interface/index.html" width="250" height="150"></iframe>\n\nFor more details, please see the following:\n\n- [Graphics class draw rect interface](graphics_draw_rect.md)\n- [Graphics class draw round rect interface](graphics_draw_round_rect.md)\n- [Graphics class draw circle interface](graphics_draw_circle.md)\n- [Graphics class draw ellipse interface](graphics_draw_ellipse.md)\n- [Graphics class move to and line to interfaces](graphics_move_to_and_line_to.md)\n- [Graphics class draw line interface](graphics_draw_line.md)\n- [Graphics class draw dotted line interface](graphics_draw_dotted_line.md)\n- [Graphics class draw dashed line interface](graphics_draw_dashed_line.md)\n- [Graphics class draw round dotted line interface](graphics_draw_round_dotted_line.md)\n- [Graphics class draw dash dotted line interface](graphics_draw_dash_dotted_line.md)\n- [Graphics class draw polygon interface](graphics_draw_polygon.md)':  # noqa
    '',

    '## See also':
    '',

    '- [Graphics class](graphics.md)\n- [Graphics class fill color interface](graphics_fill_color.md)\n- [Graphics class fill alpha interface](graphics_fill_alpha.md)\n- [Graphics class line color interface](graphics_line_color.md)\n- [Graphics class line alpha interface](graphics_line_alpha.md)\n- [Graphics class line thickness interface](graphics_line_thickness.md)\n- [Graphics class line dot setting interface](graphics_line_dot_setting.md)\n- [Graphics class line dash setting interface](graphics_line_dash_setting.md)\n- [Graphics class line round dot setting interface](graphics_line_round_dot_setting.md)\n- [Graphics class line dash dot setting interface](graphics_line_dash_dot_setting.md)':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: animation_interfaces_abstract.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Animation interfaces abstract':
    '',

    'This page explains the animation interfaces\' abstract.':
    '',

    '## What apysc can do in its animation interfaces':
    '',

    '- You can animate each attribute, such as the coordinates, rotation, color, alpha (opacity), scale.\n- You can set the animation\'s duration in milliseconds.\n- You can set the animation\'s delay in milliseconds.\n- You can use many easing settings, such as the `EASE_IN_CUBIC`\\, `EASE_OUT_QUINT`\\, and `EASE_IN_OUT_BOUNCE`\\.\n- You can control the target instance\'s animation with the pause, play, reset, finish, reverse, time interfaces.\n- You can set the parallel animation or sequential animation.\n- Animation complete event is supported.':  # noqa
    '',

    '## Examples of each attribute animation':
    '',

    'This section will show each attribute animation example:\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=550, stage_height=550, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nsprite.graphics.line_style(color=\'#fff\', thickness=1)\n\nDURATION: int = 1000\nDELAY: int = 500\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_x_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the x-animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_x(\n        x=50, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_x_animation_complete_2).start()\n\n\ndef on_x_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the x-animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_x(\n        x=100, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_x_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50,\n).animation_x(\n    x=100, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_x_animation_complete_1).start()\n\n\ndef on_y_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the y-animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_y(\n        y=50, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_y_animation_complete_2).start()\n\n\ndef on_y_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the y-animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_y(\n        y=100, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_y_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=150, y=50, width=50, height=50,\n).animation_y(\n    y=100, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_y_animation_complete_1).start()\n\n\ndef on_cx_animation_complete_1(\n        e: ap.AnimationEvent[ap.Circle], options: dict) -> None:\n    """\n    The handler that the center-x animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_x(\n        x=275, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_cx_animation_complete_2).start()\n\n\ndef on_cx_animation_complete_2(\n        e: ap.AnimationEvent[ap.Circle], options: dict) -> None:\n    """\n    The handler that the center-x animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_x(\n        x=325, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_cx_animation_complete_1).start()\n\n\nsprite.graphics.draw_circle(\n    x=275, y=75, radius=25,\n).animation_x(\n    x=325, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_cx_animation_complete_1).start()\n\n\ndef on_cy_animation_complete_1(\n        e: ap.AnimationEvent[ap.Circle], options: dict) -> None:\n    """\n    The handler that the center-y animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_y(\n        y=75, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_cy_animation_complete_2).start()\n\n\ndef on_cy_animation_complete_2(\n        e: ap.AnimationEvent[ap.Circle], options: dict) -> None:\n    """\n    The handler that the center-y animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_y(\n        y=25, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_cy_animation_complete_1).start()\n\n\nsprite.graphics.draw_circle(\n    x=375, y=75, radius=25,\n).animation_y(\n    y=25, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_cy_animation_complete_1).start()\n\n\ndef on_move_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that move-animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_move(\n        x=450, y=50, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_move_animation_complete_2).start()\n\n\ndef on_move_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that move-animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_move(\n        x=500, y=0, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_move_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=450, y=50, width=50, height=50,\n).animation_move(\n    x=500, y=0, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_move_animation_complete_1).start()\n\n\ndef on_width_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the width animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_width(\n        width=50, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_width_animation_complete_2).start()\n\n\ndef on_width_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the width animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_width(\n        width=100, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_width_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=150, width=50, height=50,\n).animation_width(\n    width=100, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_width_animation_complete_1).start()\n\n\ndef on_height_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the height animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_height(\n        height=50, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_height_animation_complete_2).start()\n\n\ndef on_height_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the height animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_height(\n        height=100, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_height_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=150, y=150, width=50, height=50,\n).animation_height(\n    height=100, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_height_animation_complete_1).start()\n\n\ndef on_ellipse_width_animation_complete_1(\n        e: ap.AnimationEvent[ap.Ellipse], options: dict) -> None:\n    """\n    The handler that the ellipse-width animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_width(\n        width=50, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_ellipse_width_animation_complete_2).start()\n\n\ndef on_ellipse_width_animation_complete_2(\n        e: ap.AnimationEvent[ap.Ellipse], options: dict) -> None:\n    """\n    The handler that the ellipse-width animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_width(\n        width=100, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_ellipse_width_animation_complete_1).start()\n\n\nsprite.graphics.draw_ellipse(\n    x=275, y=175, width=50, height=50,\n).animation_width(\n    width=100, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_ellipse_width_animation_complete_1).start()\n\n\ndef on_ellipse_height_animation_complete_1(\n        e: ap.AnimationEvent[ap.Ellipse], options: dict) -> None:\n    """\n    The handler that the ellipse-height animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_height(\n        height=50, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_ellipse_height_animation_complete_2).start()\n\n\ndef on_ellipse_height_animation_complete_2(\n        e: ap.AnimationEvent[ap.Ellipse], options: dict) -> None:\n    """\n    The handler that the ellipse-height animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_height(\n        height=100, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_ellipse_height_animation_complete_1).start()\n\n\nsprite.graphics.draw_ellipse(\n    x=375, y=175, width=50, height=50,\n).animation_height(\n    height=100, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_ellipse_height_animation_complete_1).start()\n\n\ndef on_fill_color_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the fill-color animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_fill_color(\n        fill_color=\'#0af\', duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_fill_color_animation_complete_2).start()\n\n\ndef on_fill_color_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the fill-color animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_fill_color(\n        fill_color=\'#f0a\', duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_fill_color_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=450, y=150, width=50, height=50,\n).animation_fill_color(\n    fill_color=\'#f0a\', duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_fill_color_animation_complete_1).start()\n\n\ndef on_fill_alpha_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the fill-alpha animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_fill_alpha(\n        alpha=1.0, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_fill_alpha_animation_complete_2).start()\n\n\ndef on_fill_alpha_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the fill-alpha animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_fill_alpha(\n        alpha=0.0, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_fill_alpha_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=250, width=50, height=50,\n).animation_fill_alpha(\n    alpha=0.0, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_fill_alpha_animation_complete_1).start()\n\n\ndef on_line_color_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the line-color animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_line_color(\n        line_color=\'#fff\', duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_line_color_animation_complete_2).start()\n\n\ndef on_line_color_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the line-color animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_line_color(\n        line_color=\'#666\', duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_line_color_animation_complete_1).start()\n\n\nsprite.graphics.line_style(color=\'#fff\', thickness=5)\nsprite.graphics.draw_rect(\n    x=150, y=250, width=50, height=50,\n).animation_line_color(\n    line_color=\'#666\', duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_line_color_animation_complete_1).start()\n\n\ndef on_line_alpha_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the line-alpha animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_line_alpha(\n        alpha=1.0, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_line_alpha_animation_complete_2).start()\n\n\ndef on_line_alpha_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the line-alpha animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_line_alpha(\n        alpha=0.0, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_line_alpha_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=250, y=250, width=50, height=50,\n).animation_line_alpha(\n    alpha=0.0, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_line_alpha_animation_complete_1).start()\nsprite.graphics.line_style(color=\'#fff\', thickness=1)\n\n\ndef on_line_thickness_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the line-thickness animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_line_thickness(\n        thickness=1, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_line_thickness_animation_complete_2).start()\n\n\ndef on_line_thickness_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the line-thickness animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_line_thickness(\n        thickness=5, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_line_thickness_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=350, y=250, width=50, height=50,\n).animation_line_thickness(\n    thickness=5, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_line_thickness_animation_complete_1).start()\n\n\ndef on_radius_animation_complete_1(\n        e: ap.AnimationEvent[ap.Circle], options: dict) -> None:\n    """\n    The handler that the radius-animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_radius(\n        radius=25, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_radius_animation_complete_2).start()\n\n\ndef on_radius_animation_complete_2(\n        e: ap.AnimationEvent[ap.Circle], options: dict) -> None:\n    """\n    The handler that the radius-animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_radius(\n        radius=50, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_radius_animation_complete_1).start()\n\n\nsprite.graphics.draw_circle(\n    x=475, y=275, radius=25,\n).animation_radius(\n    radius=50, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_radius_animation_complete_1).start()\n\n\ndef on_rotation_around_center_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rotation around the center point animation\n    calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_rotation_around_center(\n        rotation_around_center=0, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(\n        on_rotation_around_center_animation_complete_2,\n    ).start()\n\n\ndef on_rotation_around_center_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rotation around the center point animation\n    calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_rotation_around_center(\n        rotation_around_center=90, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(\n        on_rotation_around_center_animation_complete_1,\n    ).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=350, width=50, height=50,\n).animation_rotation_around_center(\n    rotation_around_center=90, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_rotation_around_center_animation_complete_1).start()\n\n\ndef on_rotation_around_point_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rotation around the specified point\n    animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_rotation_around_point(\n        rotation_around_point=0,\n        x=200, y=400, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(\n        on_rotation_around_point_animation_complete_2,\n    ).start()\n\n\ndef on_rotation_around_point_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rotation around the specified point\n    animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_rotation_around_point(\n        rotation_around_point=90,\n        x=200, y=400, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(\n        on_rotation_around_point_animation_complete_1,\n    ).start()\n\n\nsprite.graphics.draw_rect(\n    x=150, y=350, width=50, height=50,\n).animation_rotation_around_point(\n    rotation_around_point=90,\n    x=200, y=400, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_rotation_around_point_animation_complete_1).start()\n\n\ndef on_scale_x_from_center_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the scale-x from the center point animation\n    calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_scale_x_from_center(\n        scale_x_from_center=1.0, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(on_scale_x_from_center_animation_complete_2).start()\n\n\ndef on_scale_x_from_center_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the scale-x from the center point animation\n    calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_scale_x_from_center(\n        scale_x_from_center=0.5, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(on_scale_x_from_center_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=250, y=350, width=50, height=50,\n).animation_scale_x_from_center(\n    scale_x_from_center=0.5, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_scale_x_from_center_animation_complete_1).start()\n\n\ndef on_scale_y_from_center_animation_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the scale-y from the center point\n    animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_scale_y_from_center(\n        scale_y_from_center=1.0, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(on_scale_y_from_center_animation_2).start()\n\n\ndef on_scale_y_from_center_animation_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the scale-y from the center point\n    animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_scale_y_from_center(\n        scale_y_from_center=0.5, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(on_scale_y_from_center_animation_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=350, y=350, width=50, height=50,\n).animation_scale_y_from_center(\n    scale_y_from_center=0.5, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_scale_y_from_center_animation_1).start()\n\n\ndef on_scale_x_from_point_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the scale-x from the specified point\n    animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_scale_x_from_point(\n        scale_x_from_point=1.0, x=500, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(on_scale_x_from_point_animation_complete_2).start()\n\n\ndef on_scale_x_from_point_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the scale-x from the specified point\n    animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_scale_x_from_point(\n        scale_x_from_point=0.5, x=500, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(on_scale_x_from_point_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=450, y=350, width=50, height=50,\n).animation_scale_x_from_point(\n    scale_x_from_point=0.5, x=500, duration=DURATION, delay=DELAY,\n    easing=EASING,\n).animation_complete(on_scale_x_from_point_animation_complete_1).start()\n\n\ndef on_scale_y_from_point_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the scale-y from the specified point\n    animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_scale_y_from_point(\n        scale_y_from_point=1.0, y=500, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(on_scale_y_from_point_animation_complete_2).start()\n\n\ndef on_scale_y_from_point_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the scale-y from the specified point\n    animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_scale_y_from_point(\n        scale_y_from_point=0.5, y=500, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(on_scale_y_from_point_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=450, width=50, height=50,\n).animation_scale_y_from_point(\n    scale_y_from_point=0.5, y=500, duration=DURATION, delay=DELAY,\n).animation_complete(on_scale_y_from_point_animation_complete_1).start()\n\n\ndef on_skew_x_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the skew-x animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_skew_x(\n        skew_x=0, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_skew_x_animation_complete_2).start()\n\n\ndef on_skew_x_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the skew-x animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_skew_x(\n        skew_x=30, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_skew_x_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=150, y=450, width=50, height=50,\n).animation_skew_x(\n    skew_x=30, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_skew_x_animation_complete_1).start()\n\n\nap.save_overall_html(\n    dest_dir_path=\'animation_interfaces_abstract_each_attr/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/animation_interfaces_abstract_each_attr/index.html" width="550" height="550"></iframe>':  # noqa
    '',

    '## Easing':
    '',

    'The easing setting can be set with the `easing` argument of each animation interface. For more details: [Easing enum document](easing_enum.md)\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nap.Stage(\n    stage_width=200, stage_height=350, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nDURATION: int = 1000\nDELAY: int = 500\n\n\nclass EasingOptions(TypedDict):\n    easing: ap.Easing\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_x(\n        x=50, duration=DURATION, delay=DELAY,\n        easing=options[\'easing\'],\n    ).animation_complete(\n        on_animation_complete_2, options=options,\n    ).start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_x(\n        x=100, duration=DURATION, delay=DELAY,\n        easing=options[\'easing\'],\n    ).animation_complete(\n        on_animation_complete_1, options=options,\n    ).start()\n\n\noptions: EasingOptions = {\'easing\': ap.Easing.EASE_IN_QUINT}\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50,\n).animation_x(\n    x=100, duration=DURATION, delay=DELAY,\n    easing=options[\'easing\'],\n).animation_complete(\n    on_animation_complete_1, options=options,\n).start()\n\noptions = {\'easing\': ap.Easing.EASE_OUT_QUINT}\nsprite.graphics.draw_rect(\n    x=50, y=150, width=50, height=50,\n).animation_x(\n    x=100, duration=DURATION, delay=DELAY,\n    easing=options[\'easing\'],\n).animation_complete(\n    on_animation_complete_1, options=options,\n).start()\n\noptions = {\'easing\': ap.Easing.EASE_IN_OUT_QUINT}\nsprite.graphics.draw_rect(\n    x=50, y=250, width=50, height=50,\n).animation_x(\n    x=100, duration=DURATION, delay=DELAY,\n    easing=options[\'easing\'],\n).animation_complete(\n    on_animation_complete_1, options=options,\n).start()\n\nap.save_overall_html(\n    dest_dir_path=\'animation_interfaces_abstract_easing/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/animation_interfaces_abstract_easing/index.html" width="200" height="350"></iframe>':  # noqa
    '',

    '## X animation':
    '',

    'The `animation_x` interface sets the x-coordinate animation. For more details: [animation_x interface document](animation_x.md)\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=200, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nDURATION: int = 1000\nDELAY: int = 500\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_x(\n        x=50, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_2).start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_x(\n        x=100, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50,\n).animation_x(\n    x=100, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_animation_complete_1).start()\n\nap.save_overall_html(\n    dest_dir_path=\'animation_interfaces_abstract_x/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/animation_interfaces_abstract_x/index.html" width="200" height="150"></iframe>':  # noqa
    '',

    '## Y animation':
    '',

    'The `animation_y` interface sets the y-coordinate animation. For more details: [animation_y interface document](animation_y.md)\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=200, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nDURATION: int = 1000\nDELAY: int = 500\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_y(\n        y=50, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_2).start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_y(\n        y=100, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50,\n).animation_y(\n    y=100, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_animation_complete_1).start()\n\nap.save_overall_html(\n    dest_dir_path=\'animation_interfaces_abstract_y/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/animation_interfaces_abstract_y/index.html" width="150" height="200"></iframe>':  # noqa
    '',

    '## Move animation':
    '',

    'The `animation_move` interface sets the x- and y-coordinates animation. For more details: [animation_move interface document](animation_move.md)\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=200, stage_height=200, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nDURATION: int = 1000\nDELAY: int = 500\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_move(\n        x=50, y=50, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_2).start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_move(\n        x=100, y=100, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50,\n).animation_move(\n    x=100, y=100, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_animation_complete_1).start()\n\nap.save_overall_html(\n    dest_dir_path=\'animation_interfaces_abstract_move/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/animation_interfaces_abstract_move/index.html" width="200" height="200"></iframe>':  # noqa
    '',

    '## Width animation':
    '',

    'The `animation_width` interface sets the width animation. For more details: [animation_width and animation_height interfaces document](animation_width_and_height.md)\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=200, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nDURATION: int = 1000\nDELAY: int = 500\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_width(\n        width=50, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_2).start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_width(\n        width=100, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50,\n).animation_width(\n    width=100, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_animation_complete_1).start()\n\nap.save_overall_html(\n    dest_dir_path=\'animation_interfaces_abstract_width/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/animation_interfaces_abstract_width/index.html" width="200" height="150"></iframe>':  # noqa
    '',

    '## Height animation':
    '',

    'The `animation_height` interface sets the height animation. For more details: [animation_width and animation_height interfaces document](animation_width_and_height.md)\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=200, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nDURATION: int = 1000\nDELAY: int = 500\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_height(\n        height=50, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_2).start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_height(\n        height=100, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50,\n).animation_height(\n    height=100, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_animation_complete_1).start()\n\nap.save_overall_html(\n    dest_dir_path=\'animation_interfaces_abstract_height/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/animation_interfaces_abstract_height/index.html" width="150" height="200"></iframe>':  # noqa
    '',

    '## Fill color animation':
    '',

    'The `animation_fill_color` interface sets the fill color animation. For more details: [animation_fill_color interface document](animation_fill_color.md)\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nDURATION: int = 1000\nDELAY: int = 500\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_fill_color(\n        fill_color=\'#0af\', duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_2).start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_fill_color(\n        fill_color=\'#f0a\', duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50,\n).animation_fill_color(\n    fill_color=\'#f0a\', duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_animation_complete_1).start()\n\nap.save_overall_html(\n    dest_dir_path=\'animation_interfaces_abstract_fill_color/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/animation_interfaces_abstract_fill_color/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## Fill alpha animation':
    '',

    'The `animation_fill_alpha` interface sets the alpha (opacity) animation. For more details: [animation_fill_alpha interface document](animation_fill_alpha.md)\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nDURATION: int = 1000\nDELAY: int = 500\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_fill_alpha(\n        alpha=1.0, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_2).start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_fill_alpha(\n        alpha=0.0, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50,\n).animation_fill_alpha(\n    alpha=0.0, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_animation_complete_1).start()\n\nap.save_overall_html(\n    dest_dir_path=\'animation_interfaces_abstract_fill_alpha/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/animation_interfaces_abstract_fill_alpha/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## Line-color animation':
    '',

    'The `animation_line_color` interface sets the line color animation. For more details: [animation_line_color interface document](animation_line_color.md)\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.line_style(color=\'#0af\', thickness=5)\n\nDURATION: int = 1000\nDELAY: int = 500\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_line_color(\n        line_color=\'#0af\', duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_2).start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_line_color(\n        line_color=\'#f0a\', duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50,\n).animation_line_color(\n    line_color=\'#f0a\', duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_animation_complete_1).start()\n\nap.save_overall_html(\n    dest_dir_path=\'animation_interfaces_abstract_line_color/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/animation_interfaces_abstract_line_color/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## Line alpha animation':
    '',

    'The `animation_line_alpha` interface sets the line alpha (opacity) animation. For more details: [animation_line_alpha interface document](animation_line_alpha.md)\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.line_style(color=\'#fff\', thickness=5)\n\nDURATION: int = 1000\nDELAY: int = 500\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_line_alpha(\n        alpha=1.0, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_2).start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_line_alpha(\n        alpha=0.0, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50,\n).animation_line_alpha(\n    alpha=0.0, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_animation_complete_1).start()\n\nap.save_overall_html(\n    dest_dir_path=\'animation_interfaces_abstract_line_alpha/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/animation_interfaces_abstract_line_alpha/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## Line thickness animation':
    '',

    'The `animation_line_thickness` interface sets the line thickness (stroke width) animation. For more details: [animation_line_thickness interface document](animation_line_thickness.md)\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.line_style(color=\'#fff\', thickness=1)\n\nDURATION: int = 1000\nDELAY: int = 500\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_line_thickness(\n        thickness=1, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_2).start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_line_thickness(\n        thickness=5, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50,\n).animation_line_thickness(\n    thickness=5, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_animation_complete_1).start()\n\nap.save_overall_html(\n    dest_dir_path=\'animation_interfaces_abstract_line_thickness/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/animation_interfaces_abstract_line_thickness/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## Radius animation':
    '',

    'The `animation_radius` interface sets the radius (for instance, circle\'s radius) animation. For more details: [animation_radius interface document](animation_radius.md)\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nDURATION: int = 1000\nDELAY: int = 500\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Circle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_radius(\n        radius=25, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_2).start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Circle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_radius(\n        radius=50, duration=DURATION, delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_1).start()\n\n\nsprite.graphics.draw_circle(\n    x=75, y=75, radius=25,\n).animation_radius(\n    radius=50, duration=DURATION, delay=DELAY, easing=EASING,\n).animation_complete(on_animation_complete_1).start()\n\nap.save_overall_html(\n    dest_dir_path=\'animation_interfaces_abstract_radius/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/animation_interfaces_abstract_radius/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## Rotation animation around the center point':
    '',

    'The `animation_rotation_around_center` interface sets the rotation animation around the center point. For more details: [animation_rotation_around_center interface document](animation_rotation_around_center.md)\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nDURATION: int = 1000\nDELAY: int = 500\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_rotation_around_center(\n        rotation_around_center=0, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(on_animation_complete_2).start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_rotation_around_center(\n        rotation_around_center=90, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(on_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50,\n).animation_rotation_around_center(\n    rotation_around_center=90, duration=DURATION, delay=DELAY,\n    easing=EASING,\n).animation_complete(on_animation_complete_1).start()\n\nap.save_overall_html(\n    dest_dir_path=\'animation_interfaces_abstract_rotation_around_center/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/animation_interfaces_abstract_rotation_around_center/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## Rotation animation around the specified point':
    '',

    'The `animation_rotation_around_point` interface sets the rotation animation around the specified point. For more details: [animation_rotation_around_point interface document](animation_rotation_around_point.md)\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=200, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nDURATION: int = 1000\nDELAY: int = 500\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_rotation_around_point(\n        rotation_around_point=0, x=100, y=100, duration=DURATION,\n        delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_2).start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_rotation_around_point(\n        rotation_around_point=90, x=100, y=100, duration=DURATION,\n        delay=DELAY, easing=EASING,\n    ).animation_complete(on_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50,\n).animation_rotation_around_point(\n    rotation_around_point=90, x=100, y=100, duration=DURATION,\n    delay=DELAY, easing=EASING,\n).animation_complete(on_animation_complete_1).start()\n\nap.save_overall_html(\n    dest_dir_path=\'animation_interfaces_abstract_rotation_around_point/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/animation_interfaces_abstract_rotation_around_point/index.html" width="200" height="150"></iframe>':  # noqa
    '',

    '## Scale-x animation from the center point':
    '',

    'The `animation_scale_x_from_center` interface sets the scale-x animation from the center point. For more details: [animation_scale_x_from_center and animation_scale_x_from_center interfaces document](animation_scale_x_and_y_from_center.md)\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nDURATION: int = 1000\nDELAY: int = 500\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_scale_x_from_center(\n        scale_x_from_center=1.0, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(on_animation_complete_2).start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_scale_x_from_center(\n        scale_x_from_center=0.5, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(on_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50,\n).animation_scale_x_from_center(\n    scale_x_from_center=0.5, duration=DURATION, delay=DELAY,\n    easing=EASING,\n).animation_complete(on_animation_complete_1).start()\n\nap.save_overall_html(\n    dest_dir_path=\'animation_interfaces_abstract_scale_x_from_center/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/animation_interfaces_abstract_scale_x_from_center/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## Scale-y animation from the center point':
    '',

    'The `animation_scale_y_from_center` interface sets the scale-y animation from the center point. For more details: [animation_scale_x_from_center and animation_scale_x_from_center interfaces document](animation_scale_x_and_y_from_center.md)\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nDURATION: int = 1000\nDELAY: int = 500\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_scale_y_from_center(\n        scale_y_from_center=1.0, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(on_animation_complete_2).start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_scale_y_from_center(\n        scale_y_from_center=0.5, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(on_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50,\n).animation_scale_y_from_center(\n    scale_y_from_center=0.5, duration=DURATION, delay=DELAY,\n    easing=EASING,\n).animation_complete(on_animation_complete_1).start()\n\nap.save_overall_html(\n    dest_dir_path=\'animation_interfaces_abstract_scale_y_from_center/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/animation_interfaces_abstract_scale_y_from_center/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## Scale-x animation from the specified point':
    '',

    'The `animation_scale_x_from_point` interface sets the scale-x animation from the specified point. For more details: [animation_scale_x_from_point and animation_scale_y_from_point interfaces document](animation_scale_x_and_y_from_point.md)\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nDURATION: int = 1000\nDELAY: int = 500\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_scale_x_from_point(\n        scale_x_from_point=1.0, x=100, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(on_animation_complete_2).start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_scale_x_from_point(\n        scale_x_from_point=0.5, x=100, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(on_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50,\n).animation_scale_x_from_point(\n    scale_x_from_point=0.5, x=100, duration=DURATION, delay=DELAY,\n    easing=EASING,\n).animation_complete(on_animation_complete_1).start()\n\nap.save_overall_html(\n    dest_dir_path=\'animation_interfaces_abstract_scale_x_from_point/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/animation_interfaces_abstract_scale_x_from_point/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## Scale-y animation from the specified point':
    '',

    'The `animation_scale_y_from_point` interface sets the scale-y animation from the specified point. For more details: [animation_scale_x_from_point and animation_scale_y_from_point interfaces document](animation_scale_x_and_y_from_point.md)\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nDURATION: int = 1000\nDELAY: int = 500\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_scale_y_from_point(\n        scale_y_from_point=1.0, y=100, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(on_animation_complete_2).start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_scale_y_from_point(\n        scale_y_from_point=0.5, y=100, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(on_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50,\n).animation_scale_y_from_point(\n    scale_y_from_point=0.5, y=100, duration=DURATION, delay=DELAY,\n    easing=EASING,\n).animation_complete(on_animation_complete_1).start()\n\nap.save_overall_html(\n    dest_dir_path=\'animation_interfaces_abstract_scale_y_from_point/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/animation_interfaces_abstract_scale_y_from_point/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## Skew-x animation':
    '',

    'The `animation_skew_x` interface sets the skew-x animation. For more details: [animation_skew_x interface document](animation_skew_x.md)\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nDURATION: int = 1000\nDELAY: int = 500\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_skew_x(\n        skew_x=0, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(on_animation_complete_2).start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.target.animation_skew_x(\n        skew_x=50, duration=DURATION, delay=DELAY,\n        easing=EASING,\n    ).animation_complete(on_animation_complete_1).start()\n\n\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50,\n).animation_skew_x(\n    skew_x=50, duration=DURATION, delay=DELAY,\n    easing=EASING,\n).animation_complete(on_animation_complete_1).start()\n\nap.save_overall_html(\n    dest_dir_path=\'animation_interfaces_abstract_skew_x/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/animation_interfaces_abstract_skew_x/index.html" width="150" height="150"></iframe>':  # noqa
    '',

}
